/*
Copyright (C) 2012 Bengt Martensson.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published byto
the Free Software Foundation; either version 3 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see http://www.gnu.org/licenses/.
*/

package org.harctoolbox.IrCalc;

import javax.swing.ImageIcon;

/**
 * This class implements a GUI interactive hexadecimal/decimal calculator.
 * Really not much API in here.
 *
 */

public class HexCalc extends IrCalc {

    private static final String helpText =
            "This pane computes, for a number given in first row either as decimal or hexadecimal\n"
            + "its (one-) complement (in 8, 16, or 32 bits), its \"reverse\" (java.lang.Integer.reverse():\n"
            + "the value obtained by reversing the order of the bits in the two's complement binary\n"
            + "representation of the specified int value)."
            ;
    private static final long serialVersionUID = 1L;

    @Override
    public String getHelpText() {
        return helpText;
    }

    @Override
    public void setStatus(String message) {
        statusTextField.setText(message);
    }

    /**
     * Constructor for the HexCalc.
     *
     * @param exitOnClose If true, this is called as a program, that is to be exited when closing.
     * @param lafClassName Look-and-feel classname.
     */
    public HexCalc(boolean exitOnClose, String lafClassName) {
        super(exitOnClose, lafClassName);
     
        initComponents();

        setIconImage((new ImageIcon(getClass().getResource("/icons/crystal/24x24/apps/calc.png"))).getImage());
        
        if (exitOnClose)
            setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
    }

    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        copyPopupMenu = new javax.swing.JPopupMenu();
        copyMenuItem = new javax.swing.JMenuItem();
        copyPastePopupMenu = new javax.swing.JPopupMenu();
        cpCopyMenuItem = new javax.swing.JMenuItem();
        pasteMenuItem = new javax.swing.JMenuItem();
        hexTextField = new javax.swing.JTextField();
        reverseDecimalTextField = new javax.swing.JTextField();
        jLabel8 = new javax.swing.JLabel();
        complementDecimalTextField = new javax.swing.JTextField();
        reverseComplementHexTextField = new javax.swing.JTextField();
        jLabel14 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        reverseHexTextField = new javax.swing.JTextField();
        reverseComplementDecimalTextField = new javax.swing.JTextField();
        jLabel15 = new javax.swing.JLabel();
        jLabel9 = new javax.swing.JLabel();
        decimalTextField = new javax.swing.JTextField();
        complementHexTextField = new javax.swing.JTextField();
        jLabel7 = new javax.swing.JLabel();
        statusTextField = new javax.swing.JTextField();
        jMenuBar1 = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        exitMenuItem = new javax.swing.JMenuItem();
        helpMenu = new javax.swing.JMenu();
        aboutMenuItem = new javax.swing.JMenuItem();
        helpMenuItem = new javax.swing.JMenuItem();

        copyMenuItem.setMnemonic('C');
        copyMenuItem.setText("Copy");
        copyMenuItem.setToolTipText("Copy content of window to clipboard.");
        copyMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                copyMenuItemActionPerformed(evt);
            }
        });
        copyPopupMenu.add(copyMenuItem);

        cpCopyMenuItem.setMnemonic('C');
        cpCopyMenuItem.setText("Copy");
        cpCopyMenuItem.setToolTipText("Copy content of window to clipboard.");
        cpCopyMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cpCopyMenuItemActionPerformed(evt);
            }
        });
        copyPastePopupMenu.add(cpCopyMenuItem);

        pasteMenuItem.setMnemonic('P');
        pasteMenuItem.setText("Paste");
        pasteMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                pasteMenuItemActionPerformed(evt);
            }
        });
        copyPastePopupMenu.add(pasteMenuItem);

        setTitle("HexCalc");
        setResizable(false);

        hexTextField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        hexTextField.setText("00");
        hexTextField.setToolTipText("Enter hexadecimal number here, then press return.");
        hexTextField.setMinimumSize(new java.awt.Dimension(100, 27));
        hexTextField.setPreferredSize(new java.awt.Dimension(100, 27));
        hexTextField.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                genericCopyPasteMenu(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                genericCopyPasteMenu(evt);
            }
        });
        hexTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                hexTextFieldActionPerformed(evt);
            }
        });
        hexTextField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                hexTextFieldFocusLost(evt);
            }
        });

        reverseDecimalTextField.setEditable(false);
        reverseDecimalTextField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        reverseDecimalTextField.setText("0");
        reverseDecimalTextField.setPreferredSize(new java.awt.Dimension(100, 27));
        reverseDecimalTextField.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                genericCopyMenu(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                genericCopyMenu(evt);
            }
        });

        jLabel8.setText("Complement");

        complementDecimalTextField.setEditable(false);
        complementDecimalTextField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        complementDecimalTextField.setText("255");
        complementDecimalTextField.setPreferredSize(new java.awt.Dimension(100, 27));
        complementDecimalTextField.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                genericCopyMenu(evt);
            }
        });

        reverseComplementHexTextField.setEditable(false);
        reverseComplementHexTextField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        reverseComplementHexTextField.setText("FF");
        reverseComplementHexTextField.setPreferredSize(new java.awt.Dimension(100, 27));
        reverseComplementHexTextField.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                genericCopyMenu(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                genericCopyMenu(evt);
            }
        });

        jLabel14.setText("Reverse");

        jLabel6.setText("Decimal");

        reverseHexTextField.setEditable(false);
        reverseHexTextField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        reverseHexTextField.setText("00");
        reverseHexTextField.setMinimumSize(new java.awt.Dimension(100, 27));
        reverseHexTextField.setPreferredSize(new java.awt.Dimension(100, 27));
        reverseHexTextField.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                genericCopyMenu(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                genericCopyMenu(evt);
            }
        });

        reverseComplementDecimalTextField.setEditable(false);
        reverseComplementDecimalTextField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        reverseComplementDecimalTextField.setText("255");
        reverseComplementDecimalTextField.setPreferredSize(new java.awt.Dimension(100, 27));
        reverseComplementDecimalTextField.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                genericCopyMenu(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                genericCopyMenu(evt);
            }
        });

        jLabel15.setText("Rev.-Compl.");

        jLabel9.setText("Input");

        decimalTextField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        decimalTextField.setText("0");
        decimalTextField.setToolTipText("Enter decimal number here, then press return.");
        decimalTextField.setMinimumSize(new java.awt.Dimension(100, 27));
        decimalTextField.setPreferredSize(new java.awt.Dimension(100, 27));
        decimalTextField.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                genericCopyPasteMenu(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                genericCopyPasteMenu(evt);
            }
        });
        decimalTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                decimalTextFieldActionPerformed(evt);
            }
        });
        decimalTextField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                decimalTextFieldFocusLost(evt);
            }
        });

        complementHexTextField.setEditable(false);
        complementHexTextField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        complementHexTextField.setText("FF");
        complementHexTextField.setMinimumSize(new java.awt.Dimension(100, 27));
        complementHexTextField.setPreferredSize(new java.awt.Dimension(100, 27));
        complementHexTextField.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                genericCopyMenu(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                genericCopyMenu(evt);
            }
        });

        jLabel7.setText("Hex");

        statusTextField.setEditable(false);
        statusTextField.setText("Status Message");
        statusTextField.setToolTipText("Status message, if any");
        statusTextField.setBorder(null);
        statusTextField.setFocusable(false);

        fileMenu.setMnemonic('F');
        fileMenu.setText("File");

        exitMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_ESCAPE, 0));
        exitMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/crystal/24x24/actions/stop.png"))); // NOI18N
        exitMenuItem.setMnemonic('X');
        exitMenuItem.setText("Exit");
        exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitMenuItemActionPerformed(evt);
            }
        });
        fileMenu.add(exitMenuItem);

        jMenuBar1.add(fileMenu);

        helpMenu.setMnemonic('H');
        helpMenu.setText("Help");

        aboutMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/crystal/24x24/actions/documentinfo.png"))); // NOI18N
        aboutMenuItem.setMnemonic('A');
        aboutMenuItem.setText("About...");
        aboutMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                aboutMenuItemActionPerformed(evt);
            }
        });
        helpMenu.add(aboutMenuItem);

        helpMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F1, 0));
        helpMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/crystal/24x24/actions/agt_support.png"))); // NOI18N
        helpMenuItem.setMnemonic('H');
        helpMenuItem.setText("Help...");
        helpMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                helpMenuItemActionPerformed(evt);
            }
        });
        helpMenu.add(helpMenuItem);

        jMenuBar1.add(helpMenu);

        setJMenuBar(jMenuBar1);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(5, 5, 5)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel14)
                    .addComponent(jLabel15)
                    .addComponent(jLabel8)
                    .addComponent(jLabel9))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(reverseComplementDecimalTextField, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(reverseDecimalTextField, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(complementDecimalTextField, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(decimalTextField, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(jLabel6, javax.swing.GroupLayout.Alignment.TRAILING))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel7, javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(hexTextField, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(complementHexTextField, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(reverseHexTextField, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(reverseComplementHexTextField, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                    .addComponent(statusTextField))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(0, 0, 0)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel6)
                    .addComponent(jLabel7))
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel9)
                    .addComponent(decimalTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(hexTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(complementDecimalTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(complementHexTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel8))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(reverseDecimalTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(reverseHexTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel14))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(reverseComplementDecimalTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(reverseComplementHexTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel15))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(statusTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(5, 5, 5))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void hexTextFieldFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_hexTextFieldFocusLost
        hexTextFieldActionPerformed(null);
     }//GEN-LAST:event_hexTextFieldFocusLost

    private void hexTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_hexTextFieldActionPerformed
        try {
             int in = Integer.parseInt(hexTextField.getText(), 16);
             int noBytes = (in >= 256 || hexTextField.getText().length() > 2) ? 2 : 1;
             hexTextField.setText(String.format(noBytes == 2 ? "%04X" : "%02X", in));
             decimalTextField.setText(Integer.toString(in));
             updateHexcalc(in, noBytes);
         } catch (NumberFormatException e) {
             decimalTextField.setText("*");
             hexcalcSillyNumber(e);
         }
     }//GEN-LAST:event_hexTextFieldActionPerformed

    private void decimalTextFieldFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_decimalTextFieldFocusLost
        decimalTextFieldActionPerformed(null);
     }//GEN-LAST:event_decimalTextFieldFocusLost

    private void decimalTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_decimalTextFieldActionPerformed
        try {
             int in = Integer.parseInt(decimalTextField.getText());
             int noBytes = (in >= 256 || decimalTextField.getText().length() > 3) ? 2 : 1;
             hexTextField.setText(String.format(noBytes == 2 ? "%04X" : "%02X", in));
             updateHexcalc(in, noBytes);
         } catch (NumberFormatException e) {
             hexTextField.setText("*");
             hexcalcSillyNumber(e);
         }
     }//GEN-LAST:event_decimalTextFieldActionPerformed

    private void genericCopyPasteMenu(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_genericCopyPasteMenu
        if (evt.isPopupTrigger())
           copyPastePopupMenu.show(evt.getComponent(), evt.getX(), evt.getY());
    }//GEN-LAST:event_genericCopyPasteMenu

    private void genericCopyMenu(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_genericCopyMenu
        if (evt.isPopupTrigger())
           copyPopupMenu.show(evt.getComponent(), evt.getX(), evt.getY());
    }//GEN-LAST:event_genericCopyMenu

/*
    private void copyMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_copyMenuItemActionPerformed
        JMenuItem jmi = (JMenuItem)evt.getSource();
        JPopupMenu jpm = (JPopupMenu)jmi.getParent();
        JTextField jtf = (JTextField) jpm.getInvoker();
        (new CopyClipboardText()).toClipboard(jtf.getText());
    }//GEN-LAST:event_copyMenuItemActionPerformed

    private void cpCopyMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cpCopyMenuItemActionPerformed
        copyMenuItemActionPerformed(evt);
    }//GEN-LAST:event_cpCopyMenuItemActionPerformed

    private void pasteMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_pasteMenuItemActionPerformed
        JMenuItem jmi = (JMenuItem)evt.getSource();
        JPopupMenu jpm = (JPopupMenu)jmi.getParent();
        JTextField jtf = (JTextField) jpm.getInvoker();
        if (jtf.isEditable()) {
            jtf.setText((new CopyClipboardText()).fromClipboard());
            jtf.postActionEvent();
        }
    }//GEN-LAST:event_pasteMenuItemActionPerformed

    private void aboutMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_aboutMenuItemActionPerformed
        JOptionPane.showMessageDialog(this, aboutText, "About IrCalc", JOptionPane.INFORMATION_MESSAGE);
    }//GEN-LAST:event_aboutMenuItemActionPerformed

    private void exitMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitMenuItemActionPerformed
        dispose();
    }//GEN-LAST:event_exitMenuItemActionPerformed

    private void helpMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_helpMenuItemActionPerformed
        info(helpText);
    }//GEN-LAST:event_helpMenuItemActionPerformed
*/
    private void updateHexcalc(int in, int noBytes) {
        int comp = noBytes == 2 ? 65535 : 255;
        int rev = noBytes == 2 ? ((Integer.reverse(in) >> 16) & 65535) : ((Integer.reverse(in) >> 24) & 255);
        String hexFormat = noBytes == 2 ? "%04X" : "%02X";

        complementDecimalTextField.setText(Integer.toString(comp - in));
        complementHexTextField.setText(String.format(hexFormat, comp - in));
        reverseDecimalTextField.setText(Integer.toString(rev));
        reverseHexTextField.setText(String.format(hexFormat, rev));
        reverseComplementHexTextField.setText(String.format(hexFormat, comp-rev));
        reverseComplementDecimalTextField.setText(Integer.toString(comp-rev));
        clearStatus();
    }

    private void hexcalcSillyNumber(NumberFormatException e) {
        // Prevent multiple invocations of error()
        if (complementDecimalTextField.getText().equalsIgnoreCase("****"))
            return;

        complementDecimalTextField.setText("****");
        complementHexTextField.setText("****");
        reverseDecimalTextField.setText("****");
        reverseHexTextField.setText("****");
        error("Parse error " + e.getMessage());
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuItem aboutMenuItem;
    private javax.swing.JTextField complementDecimalTextField;
    private javax.swing.JTextField complementHexTextField;
    private javax.swing.JMenuItem copyMenuItem;
    private javax.swing.JPopupMenu copyPastePopupMenu;
    private javax.swing.JPopupMenu copyPopupMenu;
    private javax.swing.JMenuItem cpCopyMenuItem;
    private javax.swing.JTextField decimalTextField;
    private javax.swing.JMenuItem exitMenuItem;
    private javax.swing.JMenu fileMenu;
    private javax.swing.JMenu helpMenu;
    private javax.swing.JMenuItem helpMenuItem;
    private javax.swing.JTextField hexTextField;
    private javax.swing.JLabel jLabel14;
    private javax.swing.JLabel jLabel15;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JMenuItem pasteMenuItem;
    private javax.swing.JTextField reverseComplementDecimalTextField;
    private javax.swing.JTextField reverseComplementHexTextField;
    private javax.swing.JTextField reverseDecimalTextField;
    private javax.swing.JTextField reverseHexTextField;
    private javax.swing.JTextField statusTextField;
    // End of variables declaration//GEN-END:variables
}
