/*
Copyright (C) 2012 Bengt Martensson.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published byto
the Free Software Foundation; either version 3 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see http://www.gnu.org/licenses/.
*/

package org.harctoolbox.IrCalc;

import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import javax.swing.*;

/**
 * This class implements a GUI interactive hexadecimal/decimal calculator.
 * Really not much API in here.
 *
 */

public abstract class IrCalc extends javax.swing.JFrame {

    public final static String versionString = "0.2.0";

    private static final String aboutText =
            "IrCalc version " + versionString + ".\n"
            + "Copyright 2012 by Bengt Martensson.\n\n"
            + "License: GPL3.\n\n"
            + "Project home page: http://www.harctoolbox.org";

    public abstract void setStatus(String str);

    protected class CopyClipboardText implements ClipboardOwner {

        @Override
        public void lostOwnership(Clipboard c, Transferable t) {
        }

        public void toClipboard(String str) {
            Toolkit.getDefaultToolkit().getSystemClipboard().setContents(new StringSelection(str), this);
        }

        public String fromClipboard() {
            try {
                return (String) Toolkit.getDefaultToolkit().getSystemClipboard().getContents(this).getTransferData(DataFlavor.stringFlavor);
            } catch (UnsupportedFlavorException ex) {
                error(ex);
            } catch (IOException ex) {
                error(ex);
            }
            return null;
        }
    }

    //public abstract void initComponents();

    /**
     * Constructor for the IrCalc.
     *
     * @param exitOnClose If true, this is called as a program, that is to be exited when closing.
     * @param lafClassName Look-and-feel classname.
     */
    protected IrCalc(boolean exitOnClose, String lafClassName) {
        try {
            if (lafClassName != null)
                UIManager.setLookAndFeel(lafClassName);
        } catch (ClassNotFoundException ex) {
            error(ex);
        } catch (InstantiationException ex) {
            error(ex);
        } catch (IllegalAccessException ex) {
            error(ex);
        } catch (UnsupportedLookAndFeelException ex) {
            error(ex);
        }
    }
 
    protected void help(String message) {
        JOptionPane.showMessageDialog(this, message, "IrCalc help",
                JOptionPane.INFORMATION_MESSAGE,
                new ImageIcon(this.getClass().getResource("/icons/crystal/48x48/apps/khelpcenter.png")));
    }
    
    protected void about(String message) {
        JOptionPane.showMessageDialog(this, message, "About IrCalc",
                JOptionPane.INFORMATION_MESSAGE,
                new ImageIcon(this.getClass().getResource("/icons/crystal/48x48/mimetypes/info.png")));
    }

    protected void error(String message) {
        /*JOptionPane.showMessageDialog(this, message, "IrCalc error",
                JOptionPane.ERROR_MESSAGE,
                new ImageIcon(this.getClass().getResource("/icons/crystal/48x48/apps/error.png")));*/
        setStatus(message);
    }

    protected final void error(Exception ex) {
        error(ex.getMessage());
    }

    protected void clearStatus() {
        setStatus("");
    }

    protected void genericCopyPasteMenu(java.awt.event.MouseEvent evt, JPopupMenu copyPastePopupMenu) {
        if (evt.isPopupTrigger())
           copyPastePopupMenu.show(evt.getComponent(), evt.getX(), evt.getY());
    }

    protected void genericCopyMenu(java.awt.event.MouseEvent evt, JPopupMenu copyPopupMenu) {
        if (evt.isPopupTrigger())
           copyPopupMenu.show(evt.getComponent(), evt.getX(), evt.getY());
    }

    protected void copyMenuItemActionPerformed(java.awt.event.ActionEvent evt) {
        JMenuItem jmi = (JMenuItem)evt.getSource();
        JPopupMenu jpm = (JPopupMenu)jmi.getParent();
        JTextField jtf = (JTextField) jpm.getInvoker();
        (new CopyClipboardText()).toClipboard(jtf.getText());
    }

    protected void cpCopyMenuItemActionPerformed(java.awt.event.ActionEvent evt) {
        copyMenuItemActionPerformed(evt);
    }

    protected void pasteMenuItemActionPerformed(java.awt.event.ActionEvent evt) {
        JMenuItem jmi = (JMenuItem)evt.getSource();
        JPopupMenu jpm = (JPopupMenu)jmi.getParent();
        JTextField jtf = (JTextField) jpm.getInvoker();
        if (jtf.isEditable()) {
            jtf.setText((new CopyClipboardText()).fromClipboard());
            jtf.postActionEvent();
        }
    }

    protected void aboutMenuItemActionPerformed(java.awt.event.ActionEvent evt) {
        about(aboutText);
    }

    protected void exitMenuItemActionPerformed(java.awt.event.ActionEvent evt) {
        dispose();
    }

    protected abstract String getHelpText();

    protected void helpMenuItemActionPerformed(java.awt.event.ActionEvent evt) {
        help(getHelpText());
    }

    /**
     * Starts the thing from the command line.
     * @param args Currently not used
     */
    public static void main(String[] args) {
        guiExecute();
    }

    private static void guiExecute() {
        java.awt.EventQueue.invokeLater(new Runnable() {

            @Override
            public void run() {
                new HexCalc(true, null).setVisible(true);
                new TimeFrequencyCalc(true, null).setVisible(true);
            }
        });
    }
}
