/*
Copyright (C) 2012 Bengt Martensson.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published byto
the Free Software Foundation; either version 3 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see http://www.gnu.org/licenses/.
*/

package org.harctoolbox.IrCalc;

import javax.swing.ImageIcon;
import org.harctoolbox.IrpMaster.Pronto;

/**
 * This class implements a GUI interactive hexadecimal/decimal calculator.
 * Really not much API in here.
 *
 */

public class TimeFrequencyCalc extends IrCalc {

    private static final String helpText =
            "This pane computes, for a carrier frequency given either in Hz or as a Pronto code\n"
            + "(i.e., the second number in the CCF), either the time for a given number of periods\n"
            + "(entered as decimal or hexadecimal), or the number of periods as a function of the time entered.\n\n"
            + "Mathematically, time = no_periods/frequency."
            ;
    private static final long serialVersionUID = 1L;

    @Override
    protected String getHelpText() {
        return helpText;
    }

    @Override
    public void setStatus(String message) {
        statusTextField.setText(message);
    }

    /**
     * Constructor for the TimeFrequencyCalc.
     *
     * @param exitOnClose If true, this is called as a program, that is to be exited when closing.
     * @param lafClassName Look-and-feel classname.
     */
    public TimeFrequencyCalc(boolean exitOnClose, String lafClassName) {
        super(exitOnClose, lafClassName);
     
        initComponents();
        setIconImage((new ImageIcon(getClass().getResource("/icons/crystal/24x24/apps/calc.png"))).getImage());
        if (exitOnClose)
            setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

        updateFromFrequency();
        noPeriodsHexTextFieldActionPerformed(null);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        copyPopupMenu = new javax.swing.JPopupMenu();
        copyMenuItem = new javax.swing.JMenuItem();
        copyPastePopupMenu = new javax.swing.JPopupMenu();
        cpCopyMenuItem = new javax.swing.JMenuItem();
        pasteMenuItem = new javax.swing.JMenuItem();
        jLabel24 = new javax.swing.JLabel();
        frequencyTextField = new javax.swing.JTextField();
        prontocodeTextField = new javax.swing.JTextField();
        jLabel23 = new javax.swing.JLabel();
        jLabel39 = new javax.swing.JLabel();
        timeTextField = new javax.swing.JTextField();
        jLabel25 = new javax.swing.JLabel();
        noPeriodsHexTextField = new javax.swing.JTextField();
        jLabel22 = new javax.swing.JLabel();
        noPeriodsTextField = new javax.swing.JTextField();
        statusTextField = new javax.swing.JTextField();
        jMenuBar1 = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        exitMenuItem = new javax.swing.JMenuItem();
        helpMenu = new javax.swing.JMenu();
        aboutMenuItem = new javax.swing.JMenuItem();
        helpMenuItem = new javax.swing.JMenuItem();

        copyMenuItem.setMnemonic('C');
        copyMenuItem.setText("Copy");
        copyMenuItem.setToolTipText("Copy content of window to clipboard.");
        copyMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                copyMenuItemActionPerformed(evt);
            }
        });
        copyPopupMenu.add(copyMenuItem);

        cpCopyMenuItem.setMnemonic('C');
        cpCopyMenuItem.setText("Copy");
        cpCopyMenuItem.setToolTipText("Copy content of window to clipboard.");
        cpCopyMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cpCopyMenuItemActionPerformed(evt);
            }
        });
        copyPastePopupMenu.add(cpCopyMenuItem);

        pasteMenuItem.setMnemonic('P');
        pasteMenuItem.setText("Paste");
        pasteMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                pasteMenuItemActionPerformed(evt);
            }
        });
        copyPastePopupMenu.add(pasteMenuItem);

        setTitle("Time/Frequency Calculator");
        setResizable(false);

        jLabel24.setText("Time (microseconds)");

        frequencyTextField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        frequencyTextField.setText("40000");
        frequencyTextField.setToolTipText("Enter modulation frequency here, then press return.");
        frequencyTextField.setMinimumSize(new java.awt.Dimension(100, 27));
        frequencyTextField.setPreferredSize(new java.awt.Dimension(100, 27));
        frequencyTextField.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                genericCopyPasteMenu(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                genericCopyPasteMenu(evt);
            }
        });
        frequencyTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                frequencyTextFieldActionPerformed(evt);
            }
        });
        frequencyTextField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                frequencyTextFieldFocusLost(evt);
            }
        });

        prontocodeTextField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        prontocodeTextField.setText("0");
        prontocodeTextField.setToolTipText("Enter Pronto frequency code here, then press return.");
        prontocodeTextField.setMinimumSize(new java.awt.Dimension(100, 27));
        prontocodeTextField.setPreferredSize(new java.awt.Dimension(100, 27));
        prontocodeTextField.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                genericCopyPasteMenu(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                genericCopyPasteMenu(evt);
            }
        });
        prontocodeTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                prontocodeTextFieldActionPerformed(evt);
            }
        });
        prontocodeTextField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                prontocodeTextFieldFocusLost(evt);
            }
        });

        jLabel23.setText("# Periods (dec)");

        jLabel39.setText("hex");

        timeTextField.setEditable(false);
        timeTextField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        timeTextField.setText("0");
        timeTextField.setToolTipText("Duration in microseconds");
        timeTextField.setMinimumSize(new java.awt.Dimension(100, 27));
        timeTextField.setPreferredSize(new java.awt.Dimension(100, 27));
        timeTextField.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseEntered(java.awt.event.MouseEvent evt) {
                timeTextFieldMouseEntered(evt);
            }
            public void mousePressed(java.awt.event.MouseEvent evt) {
                genericCopyPasteMenu(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                genericCopyPasteMenu(evt);
            }
        });
        timeTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                timeTextFieldActionPerformed(evt);
            }
        });
        timeTextField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                timeTextFieldFocusLost(evt);
            }
        });

        jLabel25.setText("Pronto");

        noPeriodsHexTextField.setEditable(false);
        noPeriodsHexTextField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        noPeriodsHexTextField.setText("1");
        noPeriodsHexTextField.setToolTipText("Click to enable");
        noPeriodsHexTextField.setMinimumSize(new java.awt.Dimension(100, 27));
        noPeriodsHexTextField.setPreferredSize(new java.awt.Dimension(100, 27));
        noPeriodsHexTextField.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseEntered(java.awt.event.MouseEvent evt) {
                noPeriodsHexTextFieldMouseEntered(evt);
            }
            public void mousePressed(java.awt.event.MouseEvent evt) {
                genericCopyPasteMenu(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                genericCopyPasteMenu(evt);
            }
        });
        noPeriodsHexTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                noPeriodsHexTextFieldActionPerformed(evt);
            }
        });
        noPeriodsHexTextField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                noPeriodsHexTextFieldFocusLost(evt);
            }
        });

        jLabel22.setText("Freq. (Hz)");

        noPeriodsTextField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        noPeriodsTextField.setText("1");
        noPeriodsTextField.setToolTipText("Number of periods in selected frequency");
        noPeriodsTextField.setMinimumSize(new java.awt.Dimension(100, 27));
        noPeriodsTextField.setPreferredSize(new java.awt.Dimension(100, 27));
        noPeriodsTextField.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseEntered(java.awt.event.MouseEvent evt) {
                noPeriodsTextFieldMouseEntered(evt);
            }
            public void mousePressed(java.awt.event.MouseEvent evt) {
                genericCopyPasteMenu(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                genericCopyPasteMenu(evt);
            }
        });
        noPeriodsTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                noPeriodsTextFieldActionPerformed(evt);
            }
        });
        noPeriodsTextField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                noPeriodsTextFieldFocusLost(evt);
            }
        });

        statusTextField.setEditable(false);
        statusTextField.setText("Status Message");
        statusTextField.setToolTipText("Status message, if present");
        statusTextField.setBorder(null);
        statusTextField.setFocusable(false);

        fileMenu.setMnemonic('F');
        fileMenu.setText("File");

        exitMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_ESCAPE, 0));
        exitMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/crystal/24x24/actions/stop.png"))); // NOI18N
        exitMenuItem.setMnemonic('X');
        exitMenuItem.setText("Exit");
        exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitMenuItemActionPerformed(evt);
            }
        });
        fileMenu.add(exitMenuItem);

        jMenuBar1.add(fileMenu);

        helpMenu.setMnemonic('H');
        helpMenu.setText("Help");

        aboutMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/crystal/24x24/actions/documentinfo.png"))); // NOI18N
        aboutMenuItem.setMnemonic('A');
        aboutMenuItem.setText("About...");
        aboutMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                aboutMenuItemActionPerformed(evt);
            }
        });
        helpMenu.add(aboutMenuItem);

        helpMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F1, 0));
        helpMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/crystal/24x24/actions/help.png"))); // NOI18N
        helpMenuItem.setMnemonic('H');
        helpMenuItem.setText("Help...");
        helpMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                helpMenuItemActionPerformed(evt);
            }
        });
        helpMenu.add(helpMenuItem);

        jMenuBar1.add(helpMenu);

        setJMenuBar(jMenuBar1);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel24, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(statusTextField)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(layout.createSequentialGroup()
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(frequencyTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 67, javax.swing.GroupLayout.PREFERRED_SIZE)
                                    .addComponent(noPeriodsTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 67, javax.swing.GroupLayout.PREFERRED_SIZE)
                                    .addComponent(jLabel23)
                                    .addComponent(jLabel22))
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(jLabel25, javax.swing.GroupLayout.Alignment.TRAILING)
                                    .addComponent(prontocodeTextField, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, 60, javax.swing.GroupLayout.PREFERRED_SIZE)
                                    .addComponent(jLabel39, javax.swing.GroupLayout.Alignment.TRAILING)
                                    .addComponent(noPeriodsHexTextField, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, 66, javax.swing.GroupLayout.PREFERRED_SIZE)))
                            .addComponent(timeTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 67, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addGap(0, 0, Short.MAX_VALUE)))
                .addContainerGap())
        );

        layout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {frequencyTextField, noPeriodsHexTextField, noPeriodsTextField, prontocodeTextField, timeTextField});

        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap(14, Short.MAX_VALUE)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel22)
                    .addComponent(jLabel25))
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(frequencyTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(prontocodeTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(14, 14, 14)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel23)
                    .addComponent(jLabel39))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(noPeriodsTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(noPeriodsHexTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jLabel24)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(timeTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(statusTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );

        layout.linkSize(javax.swing.SwingConstants.VERTICAL, new java.awt.Component[] {frequencyTextField, prontocodeTextField});

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void selectPeriodTime(boolean selectPeriod, boolean useHex) {
        noPeriodsTextField.setEditable(selectPeriod && ! useHex);
        noPeriodsHexTextField.setEditable(selectPeriod && useHex);
        timeTextField.setEditable(!selectPeriod);
    }

    private void prontocodeTextFieldFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_prontocodeTextFieldFocusLost
        updateFromFrequencycode();
     }//GEN-LAST:event_prontocodeTextFieldFocusLost

    private void prontocodeTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_prontocodeTextFieldActionPerformed
        updateFromFrequencycode();
     }//GEN-LAST:event_prontocodeTextFieldActionPerformed

    private void timeTextFieldFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_timeTextFieldFocusLost
        timeTextFieldActionPerformed(null);
     }//GEN-LAST:event_timeTextFieldFocusLost

    private void timeTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_timeTextFieldActionPerformed
        try {
            int time = Integer.parseInt(timeTextField.getText());
            int freq = Integer.parseInt(frequencyTextField.getText());
            double periods = (((double) time) * ((double) freq)) / 1000000;
            noPeriodsTextField.setText(String.format("%.1f", periods));
            noPeriodsHexTextField.setText(String.format("%04X", Math.round(periods)));
        } catch (NumberFormatException ex) {
            error("Cannot parse `" + timeTextField.getText() + "'");
        }
     }//GEN-LAST:event_timeTextFieldActionPerformed

    private void frequencyTextFieldFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_frequencyTextFieldFocusLost
        updateFromFrequency();
     }//GEN-LAST:event_frequencyTextFieldFocusLost

    private void frequencyTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_frequencyTextFieldActionPerformed
        updateFromFrequency();
     }//GEN-LAST:event_frequencyTextFieldActionPerformed

    private void noPeriodsTextFieldFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_noPeriodsTextFieldFocusLost
        noPeriodsTextFieldActionPerformed(null);
     }//GEN-LAST:event_noPeriodsTextFieldFocusLost

    private void noPeriodsTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_noPeriodsTextFieldActionPerformed
        try {
            double noPeriods = Double.parseDouble(noPeriodsTextField.getText());
            noPeriodsHexTextField.setText(String.format("%04X", Math.round(noPeriods)));
            int freq = Integer.parseInt(frequencyTextField.getText());
            timeTextField.setText(Integer.toString((int) (1000000 * ((double) noPeriods) / (double) freq)));
        } catch (NumberFormatException ex) {
            error("Cannot parse `" + noPeriodsTextField.getText() + "'");
        }
     }//GEN-LAST:event_noPeriodsTextFieldActionPerformed

    private void genericCopyPasteMenu(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_genericCopyPasteMenu
        if (evt.isPopupTrigger())
           copyPastePopupMenu.show(evt.getComponent(), evt.getX(), evt.getY());
    }//GEN-LAST:event_genericCopyPasteMenu


    private void noPeriodsHexTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_noPeriodsHexTextFieldActionPerformed
        try {
            int noPeriods = Integer.parseInt(noPeriodsHexTextField.getText(), 16);
            noPeriodsTextField.setText(String.format("%d", noPeriods));
            int freq = Integer.parseInt(frequencyTextField.getText());
            timeTextField.setText(Integer.toString((int) (1000000 * ((double) noPeriods) / (double) freq)));
        } catch (NumberFormatException ex) {
            error("Cannot parse `" + noPeriodsHexTextField.getText() + "'");
        }
    }//GEN-LAST:event_noPeriodsHexTextFieldActionPerformed

    private void noPeriodsHexTextFieldFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_noPeriodsHexTextFieldFocusLost
        noPeriodsHexTextFieldActionPerformed(null);
    }//GEN-LAST:event_noPeriodsHexTextFieldFocusLost

    private void timeTextFieldMouseEntered(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_timeTextFieldMouseEntered
        selectPeriodTime(false, false);
    }//GEN-LAST:event_timeTextFieldMouseEntered

    private void noPeriodsTextFieldMouseEntered(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_noPeriodsTextFieldMouseEntered
        selectPeriodTime(true, false);
    }//GEN-LAST:event_noPeriodsTextFieldMouseEntered

    private void noPeriodsHexTextFieldMouseEntered(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_noPeriodsHexTextFieldMouseEntered
        selectPeriodTime(true, true);
    }//GEN-LAST:event_noPeriodsHexTextFieldMouseEntered
/*
    private void copyMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_copyMenuItemActionPerformed
        JMenuItem jmi = (JMenuItem)evt.getSource();
        JPopupMenu jpm = (JPopupMenu)jmi.getParent();
        JTextField jtf = (JTextField) jpm.getInvoker();
        (new CopyClipboardText()).toClipboard(jtf.getText());
    }//GEN-LAST:event_copyMenuItemActionPerformed

    private void cpCopyMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cpCopyMenuItemActionPerformed
        copyMenuItemActionPerformed(evt);
    }//GEN-LAST:event_cpCopyMenuItemActionPerformed

    private void pasteMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_pasteMenuItemActionPerformed
        JMenuItem jmi = (JMenuItem)evt.getSource();
        JPopupMenu jpm = (JPopupMenu)jmi.getParent();
        JTextField jtf = (JTextField) jpm.getInvoker();
        if (jtf.isEditable()) {
            jtf.setText((new CopyClipboardText()).fromClipboard());
            jtf.postActionEvent();
        }
    }//GEN-LAST:event_pasteMenuItemActionPerformed

    private void aboutMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_aboutMenuItemActionPerformed
        JOptionPane.showMessageDialog(this, aboutText, "About IrCalc", JOptionPane.INFORMATION_MESSAGE);
    }//GEN-LAST:event_aboutMenuItemActionPerformed

    private void exitMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitMenuItemActionPerformed
        dispose();
    }//GEN-LAST:event_exitMenuItemActionPerformed

    private void helpMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_helpMenuItemActionPerformed
        info(helpText);
    }//GEN-LAST:event_helpMenuItemActionPerformed
*/
    private void updateFromFrequency() {
        try {
            int freq = Integer.parseInt(frequencyTextField.getText());
            prontocodeTextField.setText(Pronto.formatInteger(Pronto.getProntoCode(freq)));
            updateFromFrequency(freq);
        } catch (NumberFormatException ex) {
            error("Cannot parse `" + frequencyTextField.getText() + "'");
        }
    }

    private void updateFromFrequencycode() {
        try {
            int freq = (int) Pronto.getFrequency(Integer.parseInt(prontocodeTextField.getText(), 16));
            frequencyTextField.setText(Integer.toString(freq));
            updateFromFrequency(freq);
        } catch (NumberFormatException ex) {
            error("Cannot parse `" + prontocodeTextField.getText() + "'");
        }
    }

    private void updateFromFrequency(int freq) {
        int time = Integer.parseInt(timeTextField.getText());
        noPeriodsTextField.setText(String.format("%.1f", (time * freq) / 1000000.0));
        clearStatus();
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuItem aboutMenuItem;
    private javax.swing.JMenuItem copyMenuItem;
    private javax.swing.JPopupMenu copyPastePopupMenu;
    private javax.swing.JPopupMenu copyPopupMenu;
    private javax.swing.JMenuItem cpCopyMenuItem;
    private javax.swing.JMenuItem exitMenuItem;
    private javax.swing.JMenu fileMenu;
    private javax.swing.JTextField frequencyTextField;
    private javax.swing.JMenu helpMenu;
    private javax.swing.JMenuItem helpMenuItem;
    private javax.swing.JLabel jLabel22;
    private javax.swing.JLabel jLabel23;
    private javax.swing.JLabel jLabel24;
    private javax.swing.JLabel jLabel25;
    private javax.swing.JLabel jLabel39;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JTextField noPeriodsHexTextField;
    private javax.swing.JTextField noPeriodsTextField;
    private javax.swing.JMenuItem pasteMenuItem;
    private javax.swing.JTextField prontocodeTextField;
    private javax.swing.JTextField statusTextField;
    private javax.swing.JTextField timeTextField;
    // End of variables declaration//GEN-END:variables
}
